local Map = class("Map")

function Map:initialize(width,height,floornum)
	local random = love.math.random
	self.WIDTH, self.HEIGHT, self.FLOOR = width, height, floornum
	for x = 1, self.WIDTH do
		self[x] = {}
		for y = 1, self.HEIGHT do
			local cell = {}
			local point = random()
			if point > 0.995 then cell.alternate = true end
			if point > 0.7 then
				cell.terrain = "#"
			else
				cell.terrain = "."
			end
			self[x][y] = cell
		end
	end
	self.spells = {}
	self.animations = {}
	self.persistantAnimations = {}
	self.queue = {}
end

function Map:update(dt)
	if #self.animations > 0 then
		for i = #self.animations, 1, -1 do
			local animation = self.animations[i]
			if animation.finished then
				table.remove(self.animations,i)
			else
				animation:update(dt,self)
			end
		end
	else
		repeat
			if self.queue[1]:update(self) then table.insert(self.queue,table.remove(self.queue,1)) end
		until(self.queue[1].id == "hero" or #self.animations > 0)
	end
end

function Map:collision(x,y)
	if self[x][y].prop and self[x][y].prop.impassable then return true
	elseif self[x][y].terrain == "#" then return true
	else return false
	end
end

function Map:line(x1,y1,x2,y2,checkEntity,checkCollision)
	local checkEntity = checkEntity or true
	local checkCollision = checkCollision or true
	local path = line(x1,y1,x2,y2)
	local result_path = {}
	for i,cell in ipairs(path) do
		local x,y = cell.x,cell.y
		table.insert(result_path, {x = x, y = y})
		local check = true
		if checkEntity then
			if self[x][y].entity then break end
		end
		if checkCollision then
			if self:collision(x,y) then break end
		end
	end
	return result_path
end

function Map:createEntity(entity,x,y)
	assert(self[x][y].entity == nil, "Tried to spawn entity where entity already exists!")
	self:insertEntity(entity,x,y)
	table.insert(self.queue,entity)
end

function Map:insertEntity(entity,x,y)
	self[x][y].entity = entity
end

function Map:moveEntity(cx,cy,tx,ty)
	local entity = self:removeEntity(cx,cy)
	self:insertEntity(entity,tx,ty)
end

function Map:removeEntity(x,y)
	local entity = self[x][y].entity
	self[x][y].entity = nil
	return entity
end

function Map:destroyEntity(x,y)
	local entity = self:removeEntity(x,y)
	for i,v in ipairs(self.queue) do
		if v.id == entity.id then
			table.remove(self.queue,i)
			break
		end
	end
end

function Map:findEntity(id)
	for x = 1, self.WIDTH do
		for y = 1, self.HEIGHT do
			if self[x][y].entity and self[x][y].entity.id == id then return x,y,self[x][y].entity end
		end
	end
end

function Map:insertItem(item,x,y)
	if not self[x][y].items then self[x][y].items = {} end
	table.insert(self[x][y].items,item)
	table.sort(self[x][y].items,sortInventory)
end

function Map:moveItem(cx,cy,tx,ty,i)
	local item = self:removeItem(cx,cy,i)
	self:addItem(item,tx,ty)
end

function Map:removeItem(x,y,i)
	if not self[x][y].items then return false end
	local i = i or 1
	local item = table.remove(self[x][y].items,i)
	if #self[x][y].items == 0 then self[x][y].items = nil end
	return item
end

function Map:draw()
	local canvas = love.graphics.newCanvas()
	love.graphics.setCanvas(canvas)
	for x in ipairs(self) do
		for y,cell in ipairs(self[x]) do
			local qx,qy,r,g,b,br,bg,bb,a = 0,0,255,255,255,0,0,0,255
			if cell.entity then
				qx,qy,r,g,b,br,bg,bb,a = cell.entity:getGraphics()
			elseif cell.prop then
			elseif cell.items then
				qx,qy,r,g,b,br,bg,bb,a = cell.items[1]:getGraphics()
			else
				if cell.terrain == "#" then
					if cell.alternate then qx,qy,r,g,b,br,bg,bb = 11,3,255,255,255,150,150,150
					else qx,qy = 12,14 end
				elseif cell.terrain == "." then qx,qy = 11,16
				end
			end
			drawTile(qx,qy,x,y,r,g,b,br,bg,bb,a)
		end
	end
	for _,animation in ipairs(self.animations) do
		animation:draw(self)
	end
	love.graphics.setColor(255,255,255,255)
	love.graphics.setCanvas()
	love.graphics.draw(canvas,290,0)
end

return Map